/**
 * ADC (analog to digital) conversion driver.
 * @author Ryan Hall, Akop Palyan
 */
component ADC {

    field ADEN: 8 = 0x80;
    field ADSC: 8 = 0x40;
    field ADFR: 8 = 0x20;
    field ADIF: 8 = 0x10;
    field ADIE: 8 = 0x08;
    field ADPS2: 8 = 0x04;
    field ADPS1: 8 = 0x02;
    field ADPS0: 8 = 0x01;

    field convFunc: function(10) = none;

    method enable() {
        // enable ADC by setting ADEN bit
        device.ADCSRA = device.ADCSRA | ADEN | ADIE;
	device.ADMUX = 0x0;
    }

    method setPrescaler(val: 3) {
        device.ADCSRA = device.ADCSRA & 0xf8 | val;
    }

    method disable() {
        // disable ADC by clearing ADEN bit
        device.ADCSRA = device.ADCSRA & ~ADEN;
    }

    method startConv(channel: int, freeRunning: boolean) {
	// start ADC conversion
       device.ADMUX = channel :: 8;
       device.ADCSRA = device.ADCSRA | ADSC;
    }

    method setConvFunc(f: function(10)) {
        convFunc = f;
    }

    method convHandler() {
        local low: 8 = device.ADCL;
        local high: 8 = device.ADCH;
        local value: 10 = (high # low) :: 10;
	convFunc(value);
    }

    method none(value: 10) { }
}
